unit ui.extension;

{$mode ObjFPC}{$H+}

interface

uses
  {$IFDEF DELPHI16_UP}
  Winapi.Windows,
  {$ELSE}
    LCLIntf,
    LCLType,
    LMessages,
  {$ENDIF}
    uCEFBrowserProcessHandler,
    uCEFInterfaces,
    uCEFProcessMessage,
    uCEFRenderProcessHandler,
    uCEFTypes,
    uCEFv8Context,
    uCEFv8Handler,
    uCEFv8Value;

type

    { TUIExtensionHandler }

    TUIExtensionHandler = class(TCefv8HandlerOwn)
    protected
        FShouldReloadSettings: boolean;
        FMyParam: string;
        function Execute(const Name: ustring; const obj: ICefv8Value; const arguments: TCefv8ValueArray; var retval: ICefv8Value; var Exception: ustring): boolean; override;

    public
        constructor Create; override;
        destructor Destroy; override;
        class procedure Register();
    end;

implementation

uses
    SysUtils,
    uCEFConstants,
    uCEFMiscFunctions;

constructor TUIExtensionHandler.Create();
begin
    inherited;
    FShouldReloadSettings := True;
end;

destructor TUIExtensionHandler.Destroy;
begin
    inherited Destroy;
end;

function TUIExtensionHandler.Execute(const Name: ustring; const obj: ICefv8Value; const arguments: TCefv8ValueArray; var retval: ICefv8Value; var Exception: ustring): boolean;
var
    TempMessage: ICefProcessMessage;
    TempFrame: ICefFrame;
    i, argsCount: integer;
    aValue, aName: string;
begin
    Result := False;

    try
        if SameText(Name, 'getPlatform') then
        begin
            retval := TCefv8ValueRef.NewString(
            {$IFDEF DARWIN}'MACOSX'{$ENDIF}
            {$IFDEF MACOSX}'MACOSX'{$ENDIF}
            {$IFDEF WINDOWS}'WINDOWS'{$ENDIF}
            {$IFDEF LINUX}'LINUX'{$ENDIF}
            );
            Result := True;
        end;

    finally
        TempMessage := nil;
    end;
end;

class procedure TUIExtensionHandler.Register;
var
    TempExtensionCode: string;

    procedure registerFunction(Name, params: string);
    begin
        TempExtensionCode :=
            TempExtensionCode +
            '  ui.' + Name + ' = function(' + params + ') {' +
            '    native function ' + Name + '(' + params + ');' +
            '    return ' + Name + '(' + params + ');' +
            '  };';
    end;

    procedure registerProcedure(Name, params: string);
    begin
        TempExtensionCode :=
            TempExtensionCode +
            '  ui.' + Name + ' = function(' + params + ') {' +
            '    native function ' + Name + '(' + params + ');' +
            '    ' + Name + '(' + params + ');' +
            '  };';
    end;

var
    TempHandler: ICefv8Handler;
begin
    CefDebugLog('AAAAAAAAAAAAa JavaScript extension registered successfully!');
    TempExtensionCode := 'var ui;' +
        'if (!ui)' +
        '  ui = {};' +
        '(function() {';


    registerFunction('getPlatform', '');

    TempExtensionCode :=
        TempExtensionCode + '})();';

    try
        TempHandler := TUIExtensionHandler.Create;

        if CefRegisterExtension('v8/ui', TempExtensionCode, TempHandler) then
        begin
            {$IFDEF DEBUG}CefDebugLog('JavaScript extension registered successfully!'){$ENDIF}
        end else begin
            {$IFDEF DEBUG}CefDebugLog('There was an error registering the JavaScript extension!'){$ENDIF}
        end;
    finally
        TempHandler := nil;
    end;
end;

end.
